@@ -1,4 +1,5 @@
 import time
+import json
 import base64
 import binascii
 import string
@@ -76,10 +77,11 @@ def s12_decrypt(data, password):
     return cipher.decrypt(data)
 
 class MappingCipherBase(object):
-    def __init__(self, password):
-        self.password = password
-        self.randomGenerator = randomutils.Random(self.password)
-        self.seeds = self.get_seeds()
+    def __init__(self, password=None):
+        self.seeds = self.try_to_load_seeds(password)
+        if not self.seeds:
+            self.randomGenerator = randomutils.Random(password)
+            self.seeds = self.get_seeds()
         self.encrypt_mapping = self.get_encrypt_mapping()
         self.decrypt_mapping = self.get_decrypt_mapping()
 
@@ -127,6 +129,26 @@ class MappingCipherBase(object):
                 raise DecryptFailed()
         return result
 
+    def dumps(self):
+        seeds = [binascii.hexlify(x).decode() for x in self.seeds]
+        data = json.dumps(seeds)
+        data = binascii.hexlify(data.encode()).decode()
+        return data
+
+    @classmethod
+    def loads(cls, data):
+        return cls(password=data)
+
+    @classmethod
+    def try_to_load_seeds(cls, data):
+        try:
+            data = binascii.unhexlify(data.encode())
+            seeds = json.loads(data)
+            seeds = [binascii.unhexlify(x) for x in seeds]
+            return seeds
+        except Exception:
+            return None
+
 class S1CipherCore(MappingCipherBase):
     def get_seeds(self):
         seeds = list(range(256))
@@ -411,3 +433,7 @@ class S2Cipher(CipherBase):
         "result_encoder": Utf8Encoder(),
         "cipher_core": S2CipherCore,
     }
+
+
+def make_cipher_key(password, key):
+    pass
